//---------------------------------------------------------------------------

#ifndef UaConfH
#define UaConfH
//---------------------------------------------------------------------------

#include <string>
#include <vector>
#include <list>
#include <assert.h>
#include <mem.h>

class UaConf
{
public:
	struct Account
	{
		enum TRANSPORT_TYPE
		{
			TRANSPORT_UDP = 0,
			TRANSPORT_TCP,
			TRANSPORT_LIMITER
		} transport;
		std::string reg_server;
		std::string user;
		std::string auth_user;
		std::string pwd;
		int reg_expires;

		std::string stun_server;

		bool operator==(const Account& right) const {
			return (reg_server == right.reg_server &&
				user == right.user &&
				auth_user == right.auth_user &&
				pwd == right.pwd &&
				transport == right.transport &&
				reg_expires == right.reg_expires &&
				stun_server == right.stun_server
				);
		}
		bool operator!=(const Account& right) const {
			return !(*this == right);
		}
		Account():
			transport(TRANSPORT_UDP),
			reg_server("127.0.0.1:5060"),
			user("3001"),
			pwd("3001"),
			reg_expires(0)	// no registration in default configuration
		{
		}
		const char* const getTransportStr(void) const {
			switch (transport)
			{
			case TRANSPORT_UDP:
				return "udp";
			case TRANSPORT_TCP:
				return "tcp";
			default:
				assert(!"Unhandled cfg transport type");
				return "???";
			}
		}
	};
	std::vector<Account> accounts;

	enum { CONTACTS_CNT = 10 };
	struct Contact {
		std::string description;
		std::string user;
		bool sub_dialog_info;
		std::list<int> btnIds;
		bool operator==(const Contact& right) const {
			return (
				description == right.description &&
				user == right.user &&
				sub_dialog_info == right.sub_dialog_info
				);
		}
		bool operator!=(const Contact& right) const {
			return !(*this == right);
		}
		Contact():
			sub_dialog_info(false)
		{
		}
	};
	std::vector<Contact> contacts;

	bool logMessages;

	static const char* modPortaudio;
	static const char* modWinwave;

	struct AudioCfg {
		char mod[32];       /**< Audio source module            */
		char dev[128];      /**< Audio source device            */
		AudioCfg(void) {
			mod[0] = '\0';
			dev[0] = '\0';
		}
		bool operator==(const AudioCfg& right) const {
			if (strcmp(mod, right.mod))
				return false;
			if (strcmp(dev, right.dev))
				return false;
			return true;
		}
		bool operator!=(const AudioCfg& right) const {
			return !(*this == right);
		}
	} audioCfgSrc, audioCfgPlay, audioCfgAlert, audioCfgPlayIntercom;

	enum Aec {
		AEC_NONE = 0,
		AEC_SPEEX,
		AEC_WEBRTC,
		AEC_LIMIT
	} aec;

	struct WebrtcAec {
		int msInSndCardBuf;
		int skew;
		bool operator==(const UaConf::WebrtcAec& right) const {
			if (msInSndCardBuf == right.msInSndCardBuf &&
				skew == right.skew)
				return true;
			return false;
		}
		bool operator!=(const UaConf::WebrtcAec& right) const {
			return !(*this == right);
		}
	} webrtcAec;

	struct Codecs {
		bool g711;
		bool gsm;
		bool g726_32;
		bool speex;
		bool operator==(const UaConf::Codecs& right) const {
			if (g711 == right.g711 &&
                g726_32 == right.g726_32 &&
				gsm == right.gsm &&
				speex == right.speex)
				return true;
			return false;
		}
		bool operator!=(const UaConf::Codecs& right) const {
			return !(*this == right);
		}
	} codecs;

	std::string local;

	bool autoAnswer;
	int autoAnswerCode;
	unsigned int autoAnswerDelayMin;
	unsigned int autoAnswerDelayMax;

	bool autoAnswerCallInfo;	///< auto-answer when Call-Info with answer-after is present in incoming INVITE
	unsigned int autoAnswerCallInfoDelayMin;	///< minimum delay when auto answering intercom/paging calls [ms]

	UaConf() {
		aec = AEC_NONE;
		memset(&codecs, 0, sizeof(codecs));
		codecs.g711 = true;
		logMessages = false;
		autoAnswer = false;
		autoAnswerCode = 200;
		autoAnswerDelayMin = 0;
		autoAnswerDelayMax = 0;
		autoAnswerCallInfo = false;
		autoAnswerCallInfoDelayMin = 0;
		contacts.resize(CONTACTS_CNT);
	}

	bool operator==(const UaConf& right) const {
		if (accounts.size() != right.accounts.size())
			return false;
		for (int i=0; i<accounts.size(); i++)
		{
			if (accounts[i] != right.accounts[i])
			{
				return false;
			}
		}
		if (contacts.size() != right.contacts.size())
			return false;
		for (int i=0; i<contacts.size(); i++)
		{
			if (contacts[i] != right.contacts[i])
			{
				return false;
			}
		}
	#if 1
		if (audioCfgSrc != right.audioCfgSrc)
			return false;
		if (audioCfgPlay != right.audioCfgPlay)
			return false;
		if (audioCfgAlert != right.audioCfgAlert)
			return false;
	#endif
		if (aec != right.aec)
			return false;
		if (local != right.local)
			return false;
		if (codecs != right.codecs)
			return false;
		return true;
	}
	bool operator!=(const UaConf& right) const {
		return !(*this == right);
	}
};

#endif
